Option Explicit

' This module is part of a toolbox to pilot OpenOffice.org from VBScript using COM Automation.
'     Copyright (C) 2007  Bernard Marcelly
' This unit is free software; you can redistribute it and/or modify it under the terms of
' the GNU Lesser General Public License as published by the Free Software Foundation;
' either version 2.1 of the License, or (at your option) any later version.
' This library is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY;
' without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
' See the GNU Lesser General Public License for more details.
' You should have received a copy of the GNU Lesser General Public License along with this
' library; if not, write to the Free Software Foundation, Inc.,
' 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


Public OpenOffice, StarDesktop, OOoIntrospection, OOoDisp

Private Const OOoErrorN = 2000


Function isNullEmpty(thisVariant)
  isNullEmpty = IsNull(thisVariant) Or IsEmpty(thisVariant) or not IsObject(thisVariant)
End Function

Function CreateUnoService(serviceName)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
Dim Result

  Set Result = OpenOffice.createInstance(serviceName)
  If isNullEmpty(Result) Then
    Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_serviceKO & serviceName
  End If
  Set CreateUnoService = Result
End Function

Function IsOpenOfficeConnected()
Dim DeskTopbis

  IsOpenOfficeConnected = False
  If isNullEmpty(OpenOffice) Then Exit Function
  On Error Resume Next
  Set DeskTopbis = OpenOffice.createInstance("com.sun.star.frame.Desktop")
  if Err.Number = 0 then
    On Error GoTo 0
    IsOpenOfficeConnected = True
  else
    Err.Clear
    On Error GoTo 0
    OpenOffice = Null
  end if
End Function

Sub ConnectOpenOffice()
  If IsOpenOfficeConnected Then Exit Sub
  On Error Resume Next
  Set OpenOffice = CreateObject("com.sun.star.ServiceManager")
  On Error GoTo 0
  If isNullEmpty(OpenOffice) Then
    Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_connectKO
  Else
    Set StarDesktop = CreateUnoService("com.sun.star.frame.Desktop")
    Set OOoIntrospection = CreateUnoService("com.sun.star.beans.Introspection")
    Set OOoDisp = CreateUnoService("com.sun.star.frame.DispatchHelper")
  End If
End Sub

Sub DisconnectOpenOffice(closeOpenOffice)
  if closeOpenOffice  then  StarDesktop.terminate
  OpenOffice = Null
  StarDesktop = Null
  OOoIntrospection = Null
  OOoDisp = Null
End Sub

Function dummyArray()
' cre un tableau vide, pour une liste vide / creates an empty array for an empty list
Dim Result()
  dummyArray = Result
End Function

Function CreateUnoStruct(structName, indexMax)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
'  indexMax = -1 returns a Uno structure
'  indexMax >= 0 returns an array of Uno structure
Dim d, Result()
  
  If indexMax < 0 Then
    On Error Resume Next
    Set CreateUnoStruct = OpenOffice.Bridge_GetStruct(structName)
    if Err.Number = 0 then
      On Error GoTo 0
    else
      Err.Clear
      On Error GoTo 0
      Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_structureKO & structName
    end if
  Else
    ReDim Result(indexMax)
    For d = 0 To indexMax
      Set Result(d) = CreateUnoStruct(structName, -1)
    Next
    CreateUnoStruct = Result
  End If
End Function

Function MakePropertyValue(PropName, PropValue)
' quivalent de la fonction de Danny Brewer
' equivalent to Danny Brewer's function
Dim Result

  Set Result = OpenOffice.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
  Result.Name = PropName
  Result.Value = PropValue
  Set MakePropertyValue = Result
End Function

Function CreateProperties(propertyList())
' fonction plus puissante que MakePropertyValue  /  function more powerful than MakePropertyValue
Dim x, xMax, y
Dim Result()

  xMax = UBound(propertyList)
  If ((xMax And 1) = 0) Or (xMax < 1) Then
    Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_nbrArgsKO
    CreateProperties = Nothing
  Else
    ReDim Result((xMax - 1) \ 2)
    x = 0
    y = 0
    Do
      Set Result(y) = OpenOffice.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
      If VarType(propertyList(x)) <> vbString Then
        Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_argRank & x & OOo_notString
      Else
        Result(y).Name = propertyList(x)
      End If
      If IsObject(propertyList(x + 1)) Then
        Set Result(y).Value = propertyList(x + 1)
      Else
        Result(y).Value = propertyList(x + 1)
      End If
      y = y + 1
      x = x + 2
    Loop Until x > xMax
    CreateProperties = Result
  End If
End Function

Function HasUnoInterfaces(thisObject, interfaceList())
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
Dim x, xMax, y, oneInterf
Dim insp, info1, info2
Dim info3, intfExists()

  HasUnoInterfaces = False
  xMax = UBound(interfaceList)
  ReDim intfExists(xMax) ' initial values : False
  On Error Resume Next
  Set insp = OOoIntrospection.inspect(thisObject)
  info1 = insp.getMethods(-1)
  if Err.Number <> 0 then
    Err.Clear
    On Error GoTo 0
    Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_inspectionKO
  end if
  For y = 0 To UBound(info1) ' scan all methods of the Uno object
    On Error Resume Next
    Set info2 = info1(y)
    Set info3 = info2.DeclaringClass
    if Err.Number <> 0 then
      Err.Clear
      On Error GoTo 0
      Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_inspectionKO
    end if
    oneInterf = info3.Name
    For x = 0 To xMax
      If oneInterf = interfaceList(x) Then
       intfExists(x) = True
      end if
    Next
  Next
  For x = 0 To xMax ' check that all interfaces requested are available
    If Not intfExists(x) Then Exit Function
  Next
  HasUnoInterfaces = True
End Function

Sub execDispatch(slotName, params())
  OOoDisp.executeDispatch StarDesktop.CurrentFrame, slotName, "", 0, params
End Sub

Sub copyToClipboard()
  Call execDispatch(".uno:Copy", dummyArray)
End Sub

Sub pasteFromClipboard()
  Call execDispatch(".uno:Paste", dummyArray)
End Sub

Function ConvertToURL(ByVal winAddr)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
Dim slow, prefix, UTF8Addr
Dim x
Dim URLprefix, sv

  URLprefix = Array("file:///", "ftp://", "news:", "http://", "mailto:", "macro:", "private:")
  slow = LCase(winAddr)
  For x = 0 To UBound(URLprefix)
    If InStr(1, slow, URLprefix(x), vbBinaryCompare) = 1 Then
      winAddr = Mid(winAddr, Len(URLprefix(x)) + 1)
      If x > 0 Then prefix = URLprefix(x) ' prefix file:/// is useless
      Exit For
    End If
  Next
  If (Len(prefix) = 0) And (InStr(1, slow, "@", vbBinaryCompare) > 0) Then
    ConvertToURL = "mailto:" & winAddr
  Else
    Set sv = CreateUnoService("com.sun.star.ucb.FileContentProvider")
    UTF8Addr = sv.getFileURLFromSystemPath("", winAddr)
    If Len(UTF8Addr) = 0 Then Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_convertToURLKO
    ConvertToURL = prefix & UTF8Addr
  End If
End Function

Function ConvertFromURL(ByVal URLaddr)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
Dim slow, prefix, winAddr, x
Dim URLprefix, sv

  URLprefix = Array("file:///", "ftp://", "news:", "http://", "mailto:", "macro:", "private:")
  slow = LCase(URLaddr)
  For x = 0 To UBound(URLprefix)
    If InStr(1, slow, URLprefix(x), vbBinaryCompare) = 1 Then
      If x > 0 Then
        URLaddr = Mid(URLaddr, Len(URLprefix(x)) + 1)
        prefix = URLprefix(x)
      End If
      Exit For
    End If
  Next
  Set sv = CreateUnoService("com.sun.star.ucb.FileContentProvider")
  winAddr = sv.getSystemPathFromFileURL(URLaddr)
  If Len(prefix) <> 0 Then ' backslash only with file:///
    winAddr = Replace(winAddr, "\", "/")
  End If
  If Len(winAddr) = 0 Then Err.Raise vbObjectError+OOoErrorN, "OpenOffice", OOo_convertFromURLKO
  ConvertFromURL = prefix & winAddr
End Function

Function OOoRGB(ByVal redV, ByVal greenV, ByVal blueV)
' quivalent de la fonction OOoBasic : RGB() / equivalent to OOoBasic function : RGB()
' ne pas utiliser  RGB() de VBscript  /  do not use RGB() from VBscript
  OOoRGB = ((redV And 255) * 65536) + ((greenV And 255) * 256) + (blueV And 255)
End Function

Function Blue(ByVal colorOOo)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
  Blue = colorOOo And 255
End Function

Function Green(ByVal colorOOo)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
  Green = ((colorOOo And 65535) \ 256) And 255
End Function

Function Red(ByVal colorOOo)
' quivalent de la fonction OOoBasic  / equivalent to OOoBasic function
  Red = ((colorOOo And 16777215) \ 65536) And 255
End Function

Sub runBasicMacro(macroName, argsList, docName)
' macroName : Library.Module.Macro
' argsList = "" if no arguments
' docName = "" if the macro is in My Macros or in OpenOffice.org Macros
  
  execDispatch "macro://" & docName & "/" & macroName & "(" & argsList & ")", dummyArray
  ' si la macro n'existe pas la routine s'excute sans erreur
  ' if the macro does not exist the routine runs without error
End Sub


Function runScript(scriptName, argsList(), language, location, resultIsObject)

Dim mspf, scriptPro, xScript

  If (language = "Basic") And (location = "user") Then location = "application"
  Set mspf = CreateUnoService("com.sun.star.script.provider.MasterScriptProviderFactory")
  Set scriptPro = mspf.createScriptProvider("")
  Set xScript = scriptPro.getScript("vnd.sun.star.script:" & scriptName & _
               "?language=" & language & "&location=" & location)
  If resultIsObject Then
    Set runScript = xScript.Invoke(argsList, dummyArray, dummyArray)
  Else
    runScript = xScript.Invoke(argsList, dummyArray, dummyArray)
  End If
End Function

Sub xray(myObject)
' vous devez avoir install Xray dans OpenOffice  /  you must have installed Xray in OpenOffice
Dim arg1(0)

  If IsObject(myObject) Then
    Set arg1(0) = myObject
  Else
    arg1(0) = myObject
  End If
  Call runScript("XrayTool._Main.Xray", arg1, "Basic", "application", False)
End Sub


